package dev.pfaff.altargui;

import org.jetbrains.annotations.Nullable;

import java.nio.charset.StandardCharsets;
import java.util.Objects;

public final class Hasher {
	private long hash;

	public void init(long seed) {
		hash = 31 * seed + Integer.hashCode(0x3eed);
	}

	public static long packString(String s) {
		long l = 0;
		var bs = s.getBytes(StandardCharsets.UTF_8);
		if (bs.length > Long.BYTES) {
			throw new IllegalArgumentException("String is too long to pack: " + s);
		}
		for (byte b : bs) {
			l = l << Byte.SIZE | Byte.toUnsignedLong(b);
		}
		return l;
	}

	public void reset() {
		hash = 0;
	}

	public long key() {
		return hash;
	}

	private void trackHashCode(long hashCode) {
		hash = 31 * hash + hashCode;
	}

	public void track(long value) {
		trackHashCode(Long.hashCode(value));
	}

	public void track(int value) {
		trackHashCode(Integer.hashCode(value));
	}

	public void track(float value) {
		trackHashCode(Float.hashCode(value));
	}

	public void track(double value) {
		trackHashCode(Double.hashCode(value));
	}

	public void track(@Nullable Object value) {
		trackHashCode(Objects.hashCode(value));
	}

	public void trackIdentity(@Nullable Object value) {
		trackHashCode(System.identityHashCode(value));
	}
}
