package dev.pfaff.altargui.widgets;

import dev.pfaff.altargui.*;
import dev.pfaff.altargui.element.CustomLayoutSingleChildWidgetInstance;
import org.jetbrains.annotations.NotNull;

/**
 * A widget that calls the provided {@code transformer} every layout.
 */
public record CustomTransformBuilder(Widget child, Transformer transformer) implements Widget {
	@Override
	public @NotNull WidgetInstance<CustomTransformBuilder> instantiate(WidgetInstantiationArgs args) {
		return new State(args);
	}

	public static final class Transform {
		public float x;
		public float y;

		private Transform(float x, float y) {
			this.x = x;
			this.y = y;
		}
	}

	@FunctionalInterface
	public interface Transformer {
		void transform(Transform transform,
					   float computedWidth, float computedHeight,
					   float availableWidth, float availableHeight);
	}

	private static final class State extends CustomLayoutSingleChildWidgetInstance<CustomTransformBuilder> {
		private Transformer transformer;

		private State(WidgetInstantiationArgs args) {
			super(args);
		}

		@Override
		public void configure(StateContext stateContext, CustomTransformBuilder widget, boolean init) {
			setChild(stateContext, widget.child);
			if (transformer != widget.transformer) {
				transformer = widget.transformer;
				markNeedsLayout();
			}
		}

		@Override
		protected void computeLayout(LayoutContext context) {
			context.layout(this, getChildElement(), (e, ctx) -> {
				e.copyAllocatedRectFrom(this);
				e.layout(ctx);
				var transform = new Transform(e.allocatedX(), e.allocatedY());
				this.transformer.transform(transform,
						e.computedWidth(),
						e.computedHeight(),
						e.allocatedWidth(),
						e.allocatedHeight());
				// TODO: should we allow overflow?
				e.setAllocatedRect(transform.x, transform.y, e.allocatedWidth(), e.allocatedHeight(), true, true);
				e.layout(ctx);
				// don't use copyComputedSizeFrom, because it checks that the allocatedX and allocatedY are equal
				// we don't want that check here
				// TODO: is this really the ideal behaviour?
				setComputedSize(e.computedWidth(), e.computedHeight());
			});
		}
	}
}
