package dev.pfaff.altargui.widgets;

import dev.pfaff.altargui.StateContext;
import dev.pfaff.altargui.Widget;
import dev.pfaff.altargui.WidgetInstance;
import dev.pfaff.altargui.WidgetInstantiationArgs;
import dev.pfaff.altargui.handlers.MouseMoveHandler;
import dev.pfaff.altargui.element.SingleChildWidgetInstance;
import it.unimi.dsi.fastutil.booleans.BooleanUnaryOperator;
import org.jetbrains.annotations.NotNull;

/**
 * The return value of {@code handler} determines whether the event will continue bubbling up the widget tree.
 */
public record MouseMoveTarget(Widget child, BooleanUnaryOperator handler) implements Widget {
	@Override
	public @NotNull WidgetInstance instantiate(WidgetInstantiationArgs args) {
		return new State(args);
	}

	private static final class State extends SingleChildWidgetInstance<MouseMoveTarget>
			implements MouseMoveHandler {
		private BooleanUnaryOperator handler;
		private boolean hovered;

		private State(WidgetInstantiationArgs args) {
			super(args);
		}

		@Override
		public void configure(StateContext stateContext, MouseMoveTarget widget, boolean init) {
			setChild(stateContext, widget.child);
			handler = widget.handler;
		}

		@Override
		public boolean mouseMoved(double mouseX, double mouseY) {
			var hovered = isInside(mouseX, mouseY);
			if (hovered != this.hovered) {
				return handler.apply(this.hovered = hovered);
			}
			return false;
		}
	}
}
