package dev.pfaff.altargui.widgets;

import dev.pfaff.altargui.StateContext;
import dev.pfaff.altargui.Widget;
import dev.pfaff.altargui.WidgetInstance;
import dev.pfaff.altargui.WidgetInstantiationArgs;
import dev.pfaff.altargui.element.SingleChildWidgetInstance;
import org.jetbrains.annotations.NotNull;

/**
 * An interface only so that records can implement it.
 */
public interface SimpleBuiltWidget extends Widget {
    /**
     * <strong>PROTECTED</strong>
     */
    Widget build();

    /**
	 * <strong>FINAL</strong>
     */
    @Override
    default @NotNull WidgetInstance<SimpleBuiltWidget> instantiate(WidgetInstantiationArgs args) {
        return new State<>(args);
    }

    static final class State<W extends SimpleBuiltWidget> extends SingleChildWidgetInstance<W> {
        private W previous = null;

        private State(WidgetInstantiationArgs args) {
            super(args);
        }

        @Override
        public void configure(StateContext stateContext, W widget, boolean init) {
            if (init || !widget.equals(this.previous)) {
                this.previous = widget;
                this.setChild(stateContext, widget.build());
            }
        }
    }
}
